;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x1e47409a85905bb8157b51da2d3c14ae . "./code.cg2js/cg-status-bar.fasl")
(#xf34ccbaf5d22d7b75043f366515fabd8 . "./code.cg2js/cg-hotspot.fasl")
(#x4b0bd8e2fe20d2b05d1899a979e1f9b7 . "./code.cg2js/cg-toolbar.fasl")
(#x2d9ddb805af3aef8487b4b42b76d74bc . "./code.cg2js/cg-clipboard-stack.fasl")
(#x28643565f7bc76f83898104a2c19f5fd . "./code.cg2js/cg-tooltip.fasl")
(#x1b126ea68ad24db5d85566221a535b48 . "./code.cg2js/cg-calendar.fasl")
(#x0a249ab1a78a5b0b3787e90e06c9cf06 . "./code.cg2js/cg-item-list.fasl")
(#xd3c8a8df8c8c4377d250f2442e103f51 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x98a74780380ffadfd9241bff9d0105b9 . "./code.cg2js/cg-trackbar.fasl")
(#xf5f61e23a0dfd734a930728fe23ae498 . "./code.cg2js/cg-class-grid.fasl")
(#x79c43e5ed24e14abf0733ae52f03f8ad . "./code.cg2js/cg-dialog-item.fasl")
(#xcac47739f56ab28238d4b564f7e6a2f3 . "./code.cg2js/cg-comtab.fasl")
(#x0631e3d4580d271ab712c20d5e47ba1e . "./code.cg2js/cg-lisp-text.fasl")
(#x5044ffe424b5bf5e4b55533485609ae2 . "./code.cg2js/cg-button.fasl")
(#xe1da11099c15392ad43f8226244b014d . "./code.cg2js/cg-printing.fasl")
(#x9299d4d2fa5cf1ef60af35f84ecd510d . "./code.cg2js/cg-pixmap-printing.fasl")
(#x71a66451b623d86d1985b0a2b7d672ce . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xee57a418d7e20cccbe1f4b7818a89e8d . "./code.cg2js/cg-pixmap-widget.fasl")
(#x96e92b27da4e30ee4acd54595c365141 . "./code.cg2js/cg-selected-object.fasl")
(#xaeb79de79c77df3b575d67a6dcc9cdf7 . "./code.cg2js/cg-text-or-combo.fasl")
(#x30617447cb408a18ffc8b61d3dabdece . "./code.cg2js/cg-graphics-context.fasl")
(#x6166408401c7a2a880446b031e851f35 . "./code.cg2js/cg-edit-in-place.fasl")
(#x7f244f0ec8d1ddce8cb260e8b359929f . "./code.cg2js/cg-string-dialog.fasl")
(#x81681feebd2cc5a66e110602caae94a5 . "./code.cg2js/cg-list-view.fasl")
(#xac4854f5858b8c4b1bca5fbbd445dcc9 . "./code.cg2js/cg-intersect.fasl")
(#x00c6a27de51c780d18458845d1407218 . "./code.cg2js/cg-text-widget.fasl")
(#x1e18a54ab6cb7e3d259a0646466383e1 . "./code.cg2js/cg-directory-dialog.fasl")
(#xc7eb877aea912aba2ba79068badbf8b8 . "./code.cg2js/cg-file-dialog.fasl")
(#x57fc48968d688b3ddd0a0bec726c3c57 . "./code.cg2js/cg-dde-utils.fasl")
(#x48a0af0ea53b3efbbf4471932f3eae03 . "./code.cg2js/cg-nodes.fasl")
(#xecf977d6e81fc495386b92bb78848c23 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x8cebc4f4234d5c54a9e80edecbb3c489 . "./code.cg2js/cg-up-down-control.fasl")
(#xe49c6fb6b3fe11d4db7a9202eb9f2ae8 . "./code.cg2js/cg-get-position.fasl")
(#x2b47edfe8940294631352e30889a67b2 . "./code.cg2js/cg-toggling-widget.fasl")
(#xdd02b51b2e006d1167449577c7f3b0a2 . "./code.cg2js/cg-acache.fasl")
(#x94633fe7c013c7b21023ee86c5264990 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x5a82f025a8604cd5f634ab95899de932 . "./code.cg2js/cg-base.fasl")
(#x1d6ebf9c4c217281b21ea79f60d2593c . "./code.cg2js/cg-drag-and-drop.fasl")
(#xafd15f88512cea2786187988be6f0372 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x73a5b1bf2e12fe5f166c5a558a162540 . "./code.cg2js/cg-property.fasl")
(#xb351881309d9942871032ba38ca0db1a . "./code.cg2js/cg-editable-text.fasl")
(#xeab66eff4c3c6ddd278cbae952da26e5 . "./code.cg2js/cg-lamp.fasl")
(#x9cab6336914959f88ea041e207d95c4d . "./code.cg2js/cg-progress-indicator.fasl")
(#xa608622fab691cd697d9f89a9b5330b5 . "./code.cg2js/cg-chart-widget.fasl")
(#x58162286f0ad7c0488011567fe4d1ea4 . "./code.cg2js/cg-common-control.fasl")
(#x38df1cd98d59eb8fff1ed21ab4b88a57 . "./code.cg2js/cg-font-dialog.fasl")
(#x1c1857e100a193015ee9ea8b5a5e68a4 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x287f1cfaf809e389d7dab6cd20260019 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x030d433c6f4403d0c40534aa8974a96a . "./code.cg2js/cg-os-window.fasl")
(#xaf6ec31c8e6eab88aa58abbe096592ba . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xc9ea91a59f53f0edbf5b015a202d23e8 . "./code.cg2js/cg-pixmap.fasl")
(#xba570231029fbe39275a3b892ecea8b4 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xb6c197722781260f8a3bdb1c74af38d6 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x1ccf4b548999e41af57f9c11a23a7193 . "./code.cg2js/cg-scaling-stream.fasl")
(#x43bca75ba47c01d1a3286cbe6a36b12e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xa5011606b475feb4303d4501c87bcf0f . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x13019f2a28747f6bcde0c04dacf5f47c . "./code.cg2js/cg-timer.fasl")
(#xa38f35e2e1b6a03b3b7ba6d1e8e8b727 . "./code.cg2js/cg-scroll-bar.fasl")
(#x837e2f03d9a2f18f6f5e6b55db911fbb . "./code.cg2js/cg-caret.fasl")
(#x891e57f8facbba3c66c9a1fe1278568a . "./code.cg2js/cg-template-string.fasl")
(#xd5eb29ab6b1474e261c9dfcbdf8b3092 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x8d668f3cb4208057ce8c894bfc6f1d36 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xda9f7395de3cb74163360402f7e2d2f9 . "./code.cg2js/cg-combo-box.fasl")
(#x99eb1a812b52e5898b0c23b4829d0d8e . "./code.cg2js/cg-text-edit-pane.fasl")
(#x941485d03741f5f6f0469adbead11888 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x90a6d51960f862ee5d8cde3d984be91d . "./code.cg2js/cg-menu-tooltip.fasl")
(#x06b196cd86aae170a2a466b8fae66120 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x0904598dba1a29bed1536bfc50da87d7 . "./code.cg2js/cg-player.fasl")
(#x0d70e4893e5bee5f07de9cdad61befa4 . "./code.cg2js/cg-outline.fasl")
(#xa14f3d7975ddbc25e96f6973d3177996 . "./code.cg2js/cg-class-support.fasl")
(#x1b49c16ced8bc53dd55e09c0088ddeea . "./code.cg2js/cg-lettered-menu.fasl")
(#x37302e76bbbe834a05e8616f122a7d7d . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xdb9ac7560ab99d458608fb2b54c4f4b5 . "./code.cg2js/cg-fill-texture.fasl")
(#x28a90be4b2357be5aca2dc84203277fc . "./code.cg2js/cg-lisp-widget.fasl")
(#x965f2f471c819257c64aeb706f7e751a . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xb3fd6e844a15f91f7b7d1a4fe0fbcf53 . "./code.cg2js/cg-clipboard.fasl")
(#x002ec1e510a3a745b3af14044c7b1090 . "./code.cg2js/cg-wrap-string.fasl")
(#x7e552e069a0125f1fa303c9b92dde633 . "./code.cg2js/cg-os-widget.fasl")
(#x94019423f971374b0cf9c7a70e6d7879 . "./code.cg2js/cg-check-box.fasl")
(#x36f05c22ebd4f624f7559891dc6e56a7 . "./code.cg2js/cg-choice-list.fasl")
(#xfb24df7d45807b7ebcaa5c49e2ba2fd7 . "./code.cg2js/cg-layout.fasl")
(#xe2cd77ae85cf52c557d9033b100d9d94 . "./code.cg2js/cg-group-box.fasl")
(#x6b2b742d3062616380a8e0bb758a48f8 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x63ea5fbd7fe8ee2f093a241976c15414 . "./code.cg2js/cg-radio-button.fasl")
(#x53fe4d9ec2c2b36533991fb92ca30ea0 . "./code.cg2js/cg-tab-control.fasl")
(#xbc76145c3453e42fc7c294fa5bd39266 . "./code.cg2js/cg.fasl")
(#x70dd001585c824845588e66035fe68a4 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x1963ef4a604127e66412c4149fe5ff91 . "./code.cg2js/cg-dropping-outline.fasl")
(#x6bddfd2983ed914a16489f88685adf92 . "./code.cg2js/cg-get-pixmap.fasl")
(#x9bd3886fe39d805fa23c7f6abc779eed . "./code.cg2js/cg-grid-widget.fasl")
(#xb7e2f09f4775b71ea74f8f3ac4b9ad9f . "./code.cg2js/cg-utility-dialog.fasl")
(#x58be862651226d7ce820f6a3b0a449b2 . "./code.cg2js/cg-static-text.fasl")
(#x22a494ab05cbc8525f0ebc9ae21cca14 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xbcc36e565fedb3c67e2ca34e8b7887e0 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xe90839f9a1d50f18130e12276e787ae7 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xd806e7c791336c984611b45a91d94e16 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xd187264bbd4cf4753b2c4d9803937cfe . "./code.cg2js/cg-plot-widget.fasl")
(#x978535055ac3ce43dcc8abdf41e63b94 . "./code.cg2js/cg-color-dialog.fasl")
(#xdbe35e33c483d4357c14f66ec647e13c . "./code.cg2js/cg-icon-pixmap.fasl")
(#x8300f3811b87166b13c28118de5068e0 . "./code.cg2js/cg-icon.fasl")
(#xe227ea30e8189737feac147d5af807c3 . "./code.cg2js/cg-dev.fasl")
(#xdfeece35acf27c3722414fa0fc8843ef . "./code.cg2js/cg-choose-printer.fasl")
(#x1ff3189bb8c50a5c82c08b336e66741d . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd183f0a38c66b6f25d59940893c752a2 . "./code.cg2js/cg-menu.fasl")
(#x709bda8378169d409c7ce66ae919062a . "./code.cg2js/cg-yes-no-list.fasl")
(#xd8e2ff332bafd98e41c9fdc4d0874712 . "./code.cg2js/cg-palette.fasl")
(#x9a9836182521a8cc563498f905e0c838 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xf17195adc67c9bc9dcd575e3d1df094a . "./code.cg2js/cg-message-dialog.fasl")
(#x6b6f56ae779762227aa4f3f3bb3eefbf . "./code.cg2js/cg-shortcut-menu.fasl")
(#xef6c2d16c04c3236d5828d49901db8cf . "./code.cg2js/cg-split-bar.fasl")
(#xbeea78e5460d0aaf8360e19000e1b230 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x80593ee5b1ac88d9c4480f995025c43a . "./code.cg2js/cg-web-browser.fasl")
(#xde3b664df7b1772d02bf8a66c521ab8f . "./code.cg2js/cg-project-window.fasl")
(#xf93baf0816b3c8d618debd1b120db263 . "./code.cg2js/cg-picture-widget.fasl")
(#xf8dfed42b851edf69fb209d73ace076e . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x18944b8d90605b6ae14bb00a75c95455 . "./code.cg2js/cg-object-editor.fasl")
(#x22cb903234cb2d136bd3ceae09531169 . "./code.cg2js/cg-paren-matching.fasl")
(#x69a62253b97e2c6e1fe41219eb28c288 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x8a23f0bca6037468389245f18db1f6bf . "./code.cg2js/cg-drawable.fasl")
(#xc5b24749143927aec819de0ef4163ee1 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x3a0aca2a5ca90268bbe6a4437953a38d . "./code.cg2js/cg-html-widget.fasl")
(#x8064d5c78652a9aaefcf14a156d17879 . "./code.cg2js/cg-curve.fasl")
(#x2226852e1b6c1682f799135a79c55fdf . "./code.cg2js/cg-yes-no-string.fasl")
(#x29162b7daf6645ff2b100a6149c68f69 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x03977b03c687a2cca752deeac3d5fce2 . "./code.cg2js/cg-html-dialog.fasl")
(#x4b10140e22a2a200e65276e0d8b63be2 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x792483faf369123dd55c6ef1fe3670ef . "./code.cg2js/cg-tray.fasl")
(#x477f0965f59e3ccc210d7c78f4ec491b . "./code.idejs/ide-apropos.fasl")
(#xdbd0c4a6979d63c644435bc46f91432e . "./code.idejs/ide-window-list.fasl")
(#xafa99ac125b47a6602fc93bf8381648d . "./code.idejs/ide-class-browser.fasl")
(#x8ce4fcaf55337bf1fd802d70d69271ff . "./code.idejs/ide-base.fasl")
(#xf7d050cf2a6ef5e762d31d3035459d89 . "./code.idejs/ide-process-browser.fasl")
(#x650b579450735c659b24f6fe8da4690e . "./code.idejs/ide-acache.fasl")
(#x3db58a9916755832f217f581d9bf327e . "./code.idejs/ide-navigator.fasl")
(#x94620a0ebc5882692ea9df99d600f254 . "./code.idejs/ide-trace.fasl")
(#x46efde7dcd2bfd53b55e4c2fd63f092c . "./code.idejs/ide-project.fasl")
(#xbcc1b2b3b2bb5378dc97c73370e864e5 . "./code.idejs/ide-patch-dialog.fasl")
(#x8dc7092bdd84e1b1f26dd6311d19f155 . "./code.idejs/ide-grapher.fasl")
(#x0f435be5db37ce62ccc6c91f18ce5c7d . "./code.idejs/ide-dev.fasl")
(#x45b70444e620f703e1e48e344f984f92 . "./code.idejs/ide-keyshort.fasl")
(#x13c18a67a0a46474833346bf47bb7a23 . "./code.idejs/ide-editor.fasl")
(#xde7d2178208344ec111c60a69f397102 . "./code.idejs/ide-inspector.fasl")
(#x37b8e1d8df35eccba1ae2cca4a24ff21 . "./code.idejs/ide.fasl")
(#x023897fd6bf27073f3e15c44ecb1892d . "./code.idejs/ide-clipboard.fasl")
(#x5dcbcac81c5ab86a934658eec2ecefce . "./code.idejs/ide-find-definitions.fasl")
(#xb665fa3a8869f3a24530c8d64297a46d . "./code.idejs/ide-form.fasl")
(#xe517c5ac5e136358b58706e36deac59f . "./code.idejs/ide-intro.fasl")
(#x91eaa01c72be7bbd579287edd81044c4 . "./code.idejs/ide-find-in-files.fasl")
(#x288317a76a576dcaa7b11986a621a48e . "./code.idejs/ide-package-list.fasl")
(#x349d1d9eb585fdd413f8e80c03e7da8e . "./code.idejs/ide-class-grapher.fasl")
(#x7e8a653f63c59e56c07642baa40b4a7f . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
